package cn.zhangdeman.logger;

import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.core.Context;
import ch.qos.logback.core.status.Status;
import cn.zhangdeman.consts.RecordField;
import cn.zhangdeman.context.RuntimeContext;
import com.fasterxml.jackson.core.JsonGenerator;
import jakarta.servlet.http.HttpServletRequest;
import net.logstash.logback.composite.JsonProvider;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.io.IOException;

// 基础provider, 简化真正provider逻辑 + 实现公共方法
public abstract class BaseProvider implements JsonProvider<ILoggingEvent> {
    // 获取请求上下文
    protected RuntimeContext getCurrentRequest() {
        try {
            RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
            if (null == requestAttributes) {
                return null;
            }
            HttpServletRequest httpServletRequest = ((ServletRequestAttributes) requestAttributes).getRequest();
            return (RuntimeContext) httpServletRequest.getAttribute(RecordField.RUNTIME_THREAD_CONTEXT);
        } catch (Exception e) {
            return null; // 非 HTTP 请求环境
        }
    }

    // 写入公共字段
    protected void writeCommonField(RuntimeContext runtimeContext, JsonGenerator gen, Object logData) throws IOException {
        gen.writeNumberField(RecordField.TIMESTAMP, System.currentTimeMillis()); // 记录日志的时间
        gen.writeStringField(RecordField.REQUEST_ID, runtimeContext.getRequestId());
        gen.writeStringField(RecordField.TRACE_ID, runtimeContext.getTraceId());
        gen.writeStringField(RecordField.SERVER_IP, runtimeContext.getServerIp());
        gen.writeStringField(RecordField.SERVER_HOSTNAME, runtimeContext.getServerHostname());
        gen.writeStringField(RecordField.REQUEST_METHOD, runtimeContext.getRequestInfo().getRequestMethod());
        gen.writeStringField(RecordField.REQUEST_URI, runtimeContext.getRequestInfo().getRequestUri());
        gen.writeStringField(RecordField.REQUEST_CLIENT_IP, runtimeContext.getRequestInfo().getClientIp());
        gen.writeNumberField(RecordField.COST, System.currentTimeMillis() - runtimeContext.getStartTimeStamp()); // 代表的是从请求开始, 到打印日志这一刻, 花费了多长时间
        gen.writeStringField(RecordField.REQUEST_CONTENT_TYPE, runtimeContext.getRequestInfo().getRequestContentType() == null ? "" : runtimeContext.getRequestInfo().getRequestContentType());
        gen.writeObjectField(RecordField.CONTEXT_DATA,logData); // 除了公共字段值外的其他数据, 统一放在logData中
    }

    @Override
    public void prepareForDeferredProcessing(ILoggingEvent iLoggingEvent) {

    }

    @Override
    public void start() {

    }

    @Override
    public void stop() {

    }

    @Override
    public boolean isStarted() {
        return false;
    }

    @Override
    public void setContext(Context context) {

    }

    @Override
    public Context getContext() {
        return null;
    }

    @Override
    public void addStatus(Status status) {

    }

    @Override
    public void addInfo(String s) {

    }

    @Override
    public void addInfo(String s, Throwable throwable) {

    }

    @Override
    public void addWarn(String s) {

    }

    @Override
    public void addWarn(String s, Throwable throwable) {

    }

    @Override
    public void addError(String s) {

    }

    @Override
    public void addError(String s, Throwable throwable) {

    }
}
